@@ -44,7 +44,7 @@ static AtkObject* gail_get_accessible_for_widget (GtkWidget    *widget,
                                                   gboolean     *transient);
 static void     gail_finish_select       (GtkWidget            *widget);
 static void     gail_map_cb              (GtkWidget            *widget);
-static void     gail_show_cb             (GtkWidget            *widget);
+static void     gail_map_submenu_cb      (GtkWidget            *widget);
 static gint     gail_focus_idle_handler  (gpointer             data);
 static void     gail_focus_notify        (GtkWidget            *widget);
 static void     gail_focus_notify_when_idle (GtkWidget            *widget);
@@ -53,6 +53,7 @@ static void     gail_focus_tracker_init ();
 
 static GtkWidget* focus_widget = NULL;
 static GtkWidget* next_focus_widget = NULL;
+static GtkWidget* subsequent_focus_widget = NULL;
 static GtkWidget* focus_before_menu = NULL;
 static guint focus_notify_handler = 0;    
 
@@ -298,7 +299,7 @@ gail_finish_select (GtkWidget *widget)
 
       menu_item = GTK_MENU_ITEM (widget);
       if (menu_item->submenu &&
-          !GTK_WIDGET_VISIBLE (menu_item->submenu))
+          !GTK_WIDGET_MAPPED (menu_item->submenu))
         {
           /*
            * If the submenu is not visble, wait until it is before
@@ -308,15 +309,15 @@ gail_finish_select (GtkWidget *widget)
 
           handler_id = g_signal_handler_find (menu_item->submenu,
                                               G_SIGNAL_MATCH_FUNC,
-                                              g_signal_lookup ("show",
+                                              g_signal_lookup ("map",
                                                                GTK_TYPE_WINDOW),
                                               0,
                                               NULL,
-                                              (gpointer) gail_show_cb,
+                                              (gpointer) gail_map_submenu_cb,
                                               NULL); 
           if (!handler_id)
-            g_signal_connect (menu_item->submenu, "show",
-                              G_CALLBACK (gail_show_cb),
+            g_signal_connect (menu_item->submenu, "map",
+                              G_CALLBACK (gail_map_submenu_cb),
                               NULL);
 
           /*
@@ -359,7 +360,7 @@ gail_map_cb (GtkWidget *widget)
 }
 
 static void
-gail_show_cb (GtkWidget *widget)
+gail_map_submenu_cb (GtkWidget *widget)
 {
   if (GTK_IS_MENU (widget))
     {
@@ -387,6 +388,9 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
   if (!GTK_IS_MENU_ITEM (widget))
     return TRUE;
 
+  if (subsequent_focus_widget == widget)
+    subsequent_focus_widget = NULL;
+
   menu_shell = gtk_widget_get_parent (widget);
   if (GTK_IS_MENU_SHELL (menu_shell))
     {
@@ -499,6 +503,12 @@ gail_focus_notify (GtkWidget *widget)
       atk_focus_tracker_notify (atk_obj);
       if (atk_obj && transient)
         g_object_unref (atk_obj);
+      if (subsequent_focus_widget)
+        {
+          GtkWidget *tmp_widget = subsequent_focus_widget;
+          subsequent_focus_widget = NULL;
+          gail_focus_notify_when_idle (tmp_widget);
+        }
     }
 }
 
@@ -516,6 +526,19 @@ gail_focus_notify_when_idle (GtkWidget *widget)
           if (GTK_IS_MENU_ITEM (next_focus_widget) && !GTK_IS_MENU_ITEM (widget))
             return;
 
+          if (next_focus_widget)
+            {
+              if (GTK_IS_MENU_ITEM (next_focus_widget) && GTK_IS_MENU_ITEM (widget))
+                {
+                  if (gtk_menu_item_get_submenu (GTK_MENU_ITEM (next_focus_widget)) == gtk_widget_get_parent (widget))
+                    {
+                      if (subsequent_focus_widget)
+                        g_assert_not_reached ();
+                      subsequent_focus_widget = widget;
+                      return;
+                    } 
+                }
+            }
           g_source_remove (focus_notify_handler);
           if (next_focus_widget)
             g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
