@@ -34,6 +34,8 @@ static AtkStateSet*          gail_button_ref_state_set    (AtkObject       *obj)
 static void                  gail_button_notify_label_gtk (GObject         *obj,
                                                            GParamSpec      *pspec,
                                                            gpointer        data);
+static gint                  gail_button_label_map_gtk    (GtkWidget       *widget,
+                                                           gpointer        data);
 
 static void                  gail_button_real_initialize  (AtkObject       *obj,
                                                            gpointer        data);
@@ -270,7 +272,19 @@ gail_button_get_name (AtkObject *obj)
       child = get_label_from_button (widget, 0, FALSE);
       if (GTK_IS_LABEL (child))
         name = gtk_label_get_text (GTK_LABEL (child)); 
+      else
+        {
+          GtkImage *image;
 
+          image = get_image_from_button (widget);
+          if (GTK_IS_IMAGE (image))
+            {
+              AtkObject *atk_obj;
+
+              atk_obj = gtk_widget_get_accessible (GTK_WIDGET (image));
+              name = atk_object_get_name (atk_obj);
+            }
+        }
       return name;
     }
 }
@@ -307,13 +321,28 @@ gail_button_real_initialize (AtkObject *obj,
   label = get_label_from_button (GTK_WIDGET (data), 0, FALSE);
   if (GTK_IS_LABEL (label))
     {
-      gail_button_init_textutil (button, label);
+      if (GTK_WIDGET_MAPPED (label))
+        gail_button_init_textutil (button, label);
+      else 
+        g_signal_connect (label,
+                          "map",
+                          G_CALLBACK (gail_button_label_map_gtk),
+                          button);
     }
-
   set_role_for_button (obj, data);
 
 }
 
+static gint
+gail_button_label_map_gtk (GtkWidget *widget,
+                           gpointer data)
+{
+  GailButton *button; 
+
+  button = GAIL_BUTTON (data);
+  gail_button_init_textutil (button, widget);
+}
+
 static void
 gail_button_notify_label_gtk (GObject           *obj,
                               GParamSpec        *pspec,
@@ -380,7 +409,7 @@ gail_button_real_add_gtk (GtkContainer *container,
 
       button = GAIL_BUTTON (data);
       if (!button->textutil)
-        gail_button_init_textutil (button, label);
+        gail_button_init_textutil (button, widget);
       else
         {
           label_text = gtk_label_get_text (label);
