@@ -132,7 +132,7 @@ gail_combo_object_init (GailCombo      *combo)
 {
   combo->press_description = NULL;
   combo->old_selection = NULL;
-  combo->deselect_idle_handler = NULL;
+  combo->deselect_idle_handler = 0;
 }
 
 AtkObject* 
