@@ -372,6 +372,11 @@ find_label (GtkWidget *start,
            * The widget may claim container as parent but it may not
            * be listed as a child.
            */
+          if (!l)
+            {
+              if (GTK_IS_SCROLLED_WINDOW (parent) && GTK_IS_SCROLLBAR (start))
+                return NULL;
+            }
           g_return_val_if_fail (l, NULL);
           if (l->prev && GTK_IS_LABEL (l->prev->data))
 	    {  
@@ -432,6 +437,16 @@ gail_widget_ref_relation_set (AtkObject *obj)
           parent = gtk_widget_get_parent (widget);
           if (parent)
             label = find_label (parent, widget);
+          /*
+           * This finds relationship in Desktop Preferences Background.
+           * See bug #126973.
+           */
+          if (!label && GTK_IS_VBOX (parent) && GTK_IS_OPTION_MENU (widget))
+            {
+              parent = gtk_widget_get_parent (parent);
+              if (parent)
+                label = find_label (parent, widget);
+            }
         }
       if (label)
         {
@@ -1034,17 +1049,7 @@ static gboolean gail_widget_on_screen (GtkWidget *widget)
           widget->allocation.y + widget->allocation.height <= 0)
         return_value = FALSE;
       else 
-        {
-        /*
-         * Check whether the widget has been placed of the screen. The
-         * widget may be MAPPED as when toolbar items do not fit on the toolbar.
-         */
-          if (widget->allocation.x + widget->allocation.width <= 0 &&
-              widget->allocation.y + widget->allocation.height <= 0)
-            return_value = FALSE;
-          else 
-            return_value = TRUE;
-        }
+        return_value = TRUE;
     }
 
   return return_value;
